import Base from './base';
import { getActualPoint, getClickPoint, createSvg, createElement } from './utils';
import { onClickOutside, toDateObject } from '../ui_utils'
import dayjs from 'dayjs'

export default class Stamp extends Base {
  
  constructor ({
    container,
    annotation,
    page,
    viewport,
    scale,
    eventBus,
    layer,
    show = false
  }) {
    super({
      container,
      annotation,
      page,
      viewport,
      scale,
      eventBus,
      show
    })

    this.layer = layer
    this.hidden = true
    this.initial = false
    this.outline = null

    this.start = null
    this.end = null

    this.newStart = null
    this.newEnd = null

    this.startCircle = null
    this.endCircle = null

    this.show = show

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.render()
  }

  render () {
    for (let key in this.annotation) {
      if (key === 'stamp-type') {
        let newName = 'stampType'
        this.annotation[newName] = this.annotation[key]
        delete this.annotation[key]
      }
      if (key === 'icon' && this.annotation['icon']) {
        this.annotation['standardStampType'] = this.annotation[key]
        delete this.annotation[key]
      }
      if ((key === 'creationdate' || key === 'date') && this.annotation.type === 'stamp') {
        let newName = 'textStampSecondstring'

        const input = this.annotation[key]
        const date = toDateObject(input)
        const formattedDate = dayjs(date).format('DD/MM/YYYY HH:mm:ss')

        this.annotation[newName] = formattedDate
        delete this.annotation[key]
      }
      if (key === 'color') {
        let newName = 'textStampColor'
        if (this.annotation[key] === '#910F05') {
          this.annotation[newName] = '1'
        }
        if (this.annotation[key] === '#3F6B21') {
          this.annotation[newName] = '2'
        }
        delete this.annotation[key]
      }
    }
    if (this.annotation.stampType === 'text' && this.annotation.standardStampType) {
      let newName = 'textStampFirststring'
      this.annotation[newName] = this.annotation.standardStampType
    }

    const annotation = this.annotation

    const { start, end } = this.getActualRect(
      this.viewport,
      this.scale
    )
    this.start = start
    this.end = end

    const rect = this.calculate(start, end)

    this.ratioRate = rect.height / rect.width

    const annotationContainer = document.createElement('div')
    annotationContainer.id = annotation.name
    annotationContainer.className = 'annotation'
    annotationContainer.style.top = rect.top + 'px'
    annotationContainer.style.left = rect.left + 'px'
    annotationContainer.style.width = rect.width + 'px'
    annotationContainer.style.height = rect.height + 'px'
    annotationContainer.style.fontSize = '0'

    let imgEle = null

    if (annotation.type === 'stamp' && (annotation.textStampFirststring || annotation.stampType === 'text')) {
      imgEle = document.createElement('div')
    } else {
      imgEle =  document.createElement('img')
      imgEle.style.width = "100%"
      imgEle.style.height = "100%"
    }

    const typesArray = [
      "Approved",
      "NotApproved",
      "Completed",
      "Final",
      "Draft",
      "Confidential",
      "ForPublicRelease",
      "NotForPublicRelease",
      "ForComment",
      "Void",
      "PreliminaryResults",
      "InformationOnly",
      "Accepted",
      "Rejected",
      "Witness",
      "InitialHere",
      "SignHere",
      "revised",
      "PrivateMark#1",
      "PrivateMark#2",
      "PrivateMark#3"
    ]
    if (annotation.type === 'image' || (annotation.type === 'stamp' && !(typesArray.includes(annotation.standardStampType) || annotation.textStampFirststring) && annotation.stampType !== 'text') || (annotation.type === 'stamp' && annotation.stampType === 'image')) {
      imgEle.src = annotation.image || annotation.imageBase64
    } else if (annotation.type === 'stamp' && annotation.standardStampType && annotation.stampType !== 'text') {
      let standardStampSrc = annotation.standardStampType
      if (standardStampSrc.includes('#')) {
        standardStampSrc = standardStampSrc.replace('#', '')
      }
      if (standardStampSrc.includes('SB')) {
        standardStampSrc = standardStampSrc.replace('SB', '')
      }
      imgEle.src = './images/stamp/' + standardStampSrc + '.png'
    } else if (annotation.type === 'stamp' && (annotation.textStampFirststring || annotation.stampType === 'text')) {
      imgEle = createElement('div', {
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'center',
        width: rect.width + 'px',
        height: rect.height + 'px',
        borderRadius: 6 * this.scale + 'px',
        textAlign: 'left',
        fontFamily: 'Helvetica',
        fontWeight: '400',
        lineHeight: '16px',
        overflow: 'hidden',
        whiteSpace: 'nowrap',
        // transform: `scale(${this.scale})`,
        // transformOrigin: 'left top'
      })
      if (annotation.textStampColor === '1') {
        imgEle.style.color = '#9A1C04'
        imgEle.style.border = '1px solid #9A1C04'
        imgEle.style.background = 'linear-gradient(259deg, rgba(255, 82, 49, 0.27) 0%, rgba(255, 82, 49, 0.06) 100%)'
      } else if (annotation.textStampColor === '2') {
        imgEle.style.color = '#4F7D2A'
        imgEle.style.border = '1px solid #A0BA8E'
        imgEle.style.background = 'linear-gradient(259deg, rgba(160, 186, 142, 0.27) 0%, rgba(160, 186, 142, 0.06) 100%)'
      }

      const text = createElement('p', {
        marginLeft: 16 * (rect.width / 190) + 'px',
        fontSize: '14px',
        transform: `scale(${rect.width / 190})`,
        transformOrigin: 'left bottom'
      })
      text.innerText = annotation.textStampFirststring
      this.text = text
  
      const time = createElement('p', {
        marginLeft: 16 * (rect.width / 190) + 'px',
        fontSize: '10px',
        transform: `scale(${rect.width / 190})`,
        transformOrigin: 'left top'
      })
      time.innerText = annotation.textStampSecondstring
      this.time = time
  
      imgEle.append(text, time)
    }

    this.imgEle = imgEle

    annotationContainer.append(this.imgEle)

    this.imgEle.addEventListener('mousedown', this.handleClick.bind(this))
    this.annotationContainer = annotationContainer

    this.container.append(this.annotationContainer)

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    this.deletetButton = createSvg(
      "svg",
      {
        width: "30",
        height: "30",
        viewBox: "0 0 30 30",
        fill: "none",
        class: 'delete-button'
      },
      {
        width: '30px',
        height: '30px'
      }
    );
    this.deletetButton.addEventListener('click', this.onDelete)
    this.deletetButton.innerHTML = this.deleteSvgStr

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left - 7}px`
    outerLine.style.top = `${rect.top - 7}px`
    outerLine.style.width = `${rect.width + 7 * 2}px`
    outerLine.style.height = `${rect.height + 7 * 2}px`
    this.outerLine = outerLine

    this.moveRect = createSvg(
      "rect",
      {
        class: "move",
        'data-id': "move",
        stroke: "#1460F3",
        'stroke-width': 1,
        'fill-opacity': 0,
        width: rect.width + 8,
        height: rect.height + 8,
        x: 3,
        y: 3
      }
    );
    
    this.topLeftRect = createSvg(
      "rect",
      {
        class: "nw-resize",
        'data-id': "nw-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: 0,
        y: 0
      }
    );

    this.bottomLeftRect = createSvg(
      "rect",
      {
        class: "sw-resize",
        'data-id': "sw-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: 0,
        y: rect.height + 8
      }
    );
    
    this.bottomRightRect = createSvg(
      "rect",
      {
        class: "se-resize",
        'data-id': "se-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width + 8,
        y: rect.height + 8
      }
    );

    this.topRightRect = createSvg(
      "rect",
      {
        class: "ne-resize",
        'data-id': "ne-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width + 8,
        y: 0
      }
    );

    this.outerLine.append(this.moveRect)
    this.outerLine.append(this.topLeftRect)
    this.outerLine.append(this.bottomLeftRect)
    this.outerLine.append(this.bottomRightRect)
    this.outerLine.append(this.topRightRect)
    this.outerLineContainer.append(this.outerLine)
    this.outerLineContainer.append(this.deletetButton)
    this.initial = true

    this.eventBus.dispatch('imageChange')
    this.layer.annotationStore.creating = false
  }

  getActualRect (viewport, s,) {
    const annotation = this.annotation
    const [x1, y1, x2, y2] = annotation.rect.split(',')
    
    const start = getActualPoint(
      {
        x: x1,
        y: y1
      },
      viewport,
      s
    )
    const end = getActualPoint(
      {
        x: x2,
        y: y2
      },
      viewport,
      s
    )
    return {
      start,
      end
    }
  }

  rectCalc ({ start, end }) {
    return {
      top: start.y < end.y ? start.y : end.y,
      left: start.x < end.x ? start.x : end.x,
      width: Math.abs(end.x - start.x),
      height: Math.abs(end.y - start.y),
    };
  };

  calculate (start, end) {
    const initRect = this.rectCalc({ start, end })

    return {
      top: initRect.top,
      left: initRect.left,
      width: initRect.width,
      height: initRect.height,
    }
  }

  getInitialPoint () {
    const s = this.scale
    const startX = this.start.x
    const startY = this.start.y
    const endX = this.end.x
    const endY = this.end.y

    const { width, height, rotation  } = this.viewport;
    let x1, y1, x2, y2;
    if (rotation === 0) {
      x1 = startX / s;
      y1 = (height - startY) / s; 
      x2 = endX / s;
      y2 = (height - endY) / s;
    } else if (rotation === 90) {
      x1 = startY / s;
      y1 = startX / s;
      x2 = endY / s;
      y2 = endX / s;
    } else if (rotation === 180) {
      x1 =( width - startX) / s;
      y1 = startY / s;
      x2 = (width - endX) / s;
      y2 = endY / s;
    } else {
      x1 = (height - startY) / s
      y1 = (width - startX) / s
      x2 = (height - endY) / s
      y2 = (width - endX)  / s
    }
    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    };
  }

  updateTool () {
    if (!this.initial) return
    if (this.hidden) {
      if (this.layer.selectedElementName === this.annotation.name) {
        this.layer.selectedElementName = null
      }
      this.outerLineContainer.remove()
    } else {
      if (this.layer.selectedElementName !== this.annotation.name) {
        this.layer.selectedElementName = this.annotation.name
      }
      this.container.append(this.outerLineContainer)
      this.outerLine.addEventListener('mousedown', this.onMousedown)
      this.outerLine.addEventListener('touchstart', this.onMousedown)
    }
  }

  handleClick (event) {
    if (this.layer.tool) {
      event.preventDefault()
    }
    if (!this.hidden || this.layer.annotationStore.creating) return
    this.hidden = false
    this.updateTool()
    onClickOutside([this.annotationContainer, this.outerLine, this.deletetButton], this.handleOutside.bind(this))
  }

  handleOutside () {
    this.hidden = !this.hidden
    if (this.layer.selectedElementName === this.annotation.name) {
      this.layer.selectedElementName = null
    }

    this.outerLine.removeEventListener('mousedown', this.onMousedown)
    this.outerLine.removeEventListener('touchstart', this.onMousedown)
    this.outerLineContainer.remove()
  }

  handleMouseDown (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchstart') {
      document.body.style.overscrollBehavior = 'none';
      document.body.style.userSelect = 'none';
      document.getElementById('app').style.touchAction = 'none';
      document.querySelector('.document-container').style.overflow = 'hidden'
    }
    const operatorId = event.target.getAttribute('data-id')
  
    const { pageX, pageY } = getClickPoint(event)
    this.startState = {
      operator: operatorId,
      clickX: pageX,
      clickY: pageY,
    }
   
    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
    document.addEventListener('touchmove', this.onMousemove)
    document.addEventListener('touchend', this.onMouseup)
  };

  handleMouseMove (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    this.moving = true
    const { pageX, pageY } = getClickPoint(event)
    const start = this.start
    const end = this.end
    const startState = this.startState

    const { width, height } = this.viewport
    if (startState.operator === 'nw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.end.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(start.y - 5, y1)
      y1 = Math.max(0, y1)

      if (this.ratioRate) {
        let moveX = x1 - this.start.x
        let moveY = moveX * this.ratioRate
        y1 = this.end.y + moveY
        
        if (y1 < 0 || x1 < 0) return
      }

      this.update({
        start: { x: x1, y: start.y },
        end: { x: end.x, y: y1 },
      });
     
    } else if (startState.operator === 'w-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)

      this.update({
        start: { x: x1, y: start.y },
        end,
      });
    } else if (startState.operator === 'sw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(height, y1)
      y1 = Math.max(end.y + 5, y1)

      if (this.ratioRate) {
        let moveX = x1 - this.start.x
        let moveY = moveX * this.ratioRate
        y1 = this.start.y - moveY
        if (y1 >= height || x1 < 0) return
      }

      this.update({
        start: { x: x1, y: y1 },
        end,
      });
    } else if (startState.operator === 's-resize') {
      let y1 = pageY - (startState.clickY - this.start.y);
      y1 = Math.min(height, y1)
      y1 = Math.max(end.y + 5, y1)
      this.update({
        start: { x: start.x, y: y1 },
        end,
      });
    } else if (startState.operator === 'se-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.start.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(height, y)
      y = Math.max(end.y + 5, y)

      if (this.ratioRate) {
        let moveX = this.start.x - x
        let moveY = moveX * this.ratioRate
        y = this.end.y - moveY

        if (y >= height || x >= width) return
      }

      this.update({
        start: { x: start.x, y },
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'e-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      this.update({
        start,
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'ne-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.end.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(start.y - 5, y)
      y = Math.max(0, y)

      if (this.ratioRate) {
        let moveX = this.start.x - x
        let moveY = moveX * this.ratioRate
        y = this.start.y + moveY

        if (y <=0 || x >= width) return
      }

      this.update({
        start,
        end: { x, y },
      });
    } else if (startState.operator === 'n-resize') {
      let y = pageY - (startState.clickY - this.end.y);
      y = Math.min(start.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start,
        end: { x: end.x, y },
      });
    } else if (startState.operator === 'move') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      let x2 = pageX - (startState.clickX - this.end.x);
      let y2 = pageY - (startState.clickY - this.end.y);

      const rect = {
        width: Math.abs(start.x -end.x),
        height: Math.abs(start.y -end.y)
      }
      if (x1 < x2) {
        x1 = Math.max(0, x1)
        x1 = Math.min(width - rect.width, x1)
        x2 = Math.max(rect.width, x2)
        x2 = Math.min(width, x2)
      } else {
        x2 = Math.max(0, x2)
        x2 = Math.min(width - rect.width, x2)
        x1 = Math.max(rect.width, x1)
        x1 = Math.min(width, x1)
      }

      if (y1 < y2) {
        y1 = Math.max(0, y1)
        y1 = Math.min(height - rect.height, y1)
        y2 = Math.max(rect.height, y2)
        y2 = Math.min(height, y2)
      } else {
        y2 = Math.max(0, y2)
        y2 = Math.min(height - rect.height, y2)
        y1 = Math.max(rect.height, y1)
        y1 = Math.min(height, y1)
      }

      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      });
    }
  }

  handleDelete (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    this.handleOutside()
    this.annotationContainer.remove()

    this.annotation.isDelete = true
    this.eventBus.dispatch('annotationChange', {
      type: 'delete',
      annotation: {
        operate: "del-annot",
        name: this.annotation.name,
        page: this.page
      }
    })
  }

  handleMouseUp (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchend') {
      document.body.style.overscrollBehavior = 'auto';
      document.body.style.userSelect = 'auto';
      document.getElementById('app').style.touchAction = 'auto';
      document.querySelector('.document-container').style.overflow = 'auto'
    }
    this.moving = false
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)
    document.removeEventListener('touchmove', this.onMousemove)
    document.removeEventListener('touchend', this.onMouseup)
    
    const { pageX, pageY } = getClickPoint(event)
    if (pageX === this.startState.clickX && pageY === this.startState.clickY) return

    this.start = this.newStart
    this.end = this.newEnd

    const annotation = this.annotation
    const { start, end } = this.getInitialPoint()

    const linePoints = start.x + ',' + start.y + ',' + end.x + ',' + end.y
    if (linePoints === annotation.linePoints) return
    annotation.linePoints = linePoints

    this.eventBus.dispatch('annotationChange', {
      type: 'modify',
      annotation: {
        operate: "mod-annot",
        name: annotation.name,
        page: this.page,
        linePoints: linePoints,
        rect: linePoints
      }
    })
  }

  update ({ start, end }) {
    const rect = this.calculate(start, end)

    this.newStart = start
    this.newEnd = end

    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'

    if (this.annotation.type === 'stamp' && (this.annotation.stampType === 'text' || this.annotation.textStampFirststring || this.annotation.textStampSecondstring)) {
      // const originWidth = this.imgEle.offsetWidth
      const ratio = rect.width / 190
      // this.imgEle.style.transform = `scale(${ratio})`
      this.imgEle.style.borderRadius = 6 * ratio + 'px'
      this.text.style.transform = `scale(${ratio})`
      this.text.style.marginLeft = 16 * ratio + 'px'
      this.time.style.transform = `scale(${ratio})`
      this.time.style.marginLeft = 16 * ratio + 'px'
    }
    this.imgEle.style.width = rect.width + 'px'
    this.imgEle.style.height = rect.height + 'px'

    this.outerLine.style.left = `${rect.left - 7}px`
    this.outerLine.style.top = `${rect.top - 7}px`
    this.outerLine.style.width = `${rect.width + 7 * 2}px`
    this.outerLine.style.height = `${rect.height + 7 * 2}px`

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    this.moveRect.setAttribute('width',  rect.width + 8)
    this.moveRect.setAttribute('height',  rect.height + 8)
    
    this.bottomLeftRect.setAttribute("y", rect.height + 8)

    this.bottomRightRect.setAttribute("x", rect.width + 8)
    this.bottomRightRect.setAttribute("y", rect.height + 8)
    
    this.topRightRect.setAttribute("x", rect.width + 8)
  }
}
