import { defineStore } from 'pinia'
import core from '@/core'

export const useViewerStore = defineStore({
  id: 'viewer',
  state: () => ({
    fullMode: false,
    currentPage: 0,
    searchStatus: false,
    scale: '',
    themeMode: 'Light',
    pageMode: 0,
    scrollMode: 'Vertical',
    pageLabels: [],
    zoomLevel: ['auto', 'page-fit', 0.5, 1, 1.25, 1.5, 2, 2.5],
    activeTab: 0,
    activeLeftPanel: 'THUMBS',
    activeLeftPanelTab: 'THUMBS',
    activeRightPanelTab: 'GENERAL',
    activeStampPanelTab: 'STANDARD',
    activeStickNote: false,
    activeActiveMeasure: false,
    activeHand: false,
    openElements: {
      header: true,
      leftPanel: false,
      rightPanel: false,
      pageSetting: false,
      stampPanel: false,
      linkPanel: false
    },
    headers: [
      {
        type: 'toggleElementButton',
        img: 'icon-header-sidebar-line',
        element: 'leftPanel',
        dataElement: 'leftPanelButton',
        title: 'Left Panel'
      },
      {
        type: 'divider',
        hidden: [
          'small-mobile'
        ]
      },
      {
        type: 'pageDisplayButton',
        dataElement: 'pageDisplayButton',
        element: 'pageDisplayButton',
        title: 'Page Display'
      },
      {
        type: 'fullScreenButton',
        dataElement: 'fullScreenButton',
        element: 'fullScreenButton',
        title: 'Full Screen'
      },
      {
        type: 'handToolButton',
        dataElement: 'handToolButton',
        element: 'handToolButton',
        title: 'Pan Tool'
      },
      {
        type: 'divider',
        hidden: [
          'small-mobile'
        ]
      },
      {
        type: 'zoomOverlay',
        dataElement: 'zoomOverlayButton',
        element: 'zoomOverlay',
        hiddenOnMobileDevice: true
      },
      {
        type: 'divider',
        hidden: [
          'small-mobile'
        ]
      },
      {
        type: 'themeMode',
        dataElement: 'themeMode',
        element: 'themeMode'
      },
      {
        type: 'stickyNoteButton',
        dataElement: 'stickyNoteButton',
        element: 'stickyNoteButton',
        hidden: false,
      },
      {
        type: 'measureButton',
        dataElement: 'measureButton',
        element: 'measureButton',
        hidden: true,
      },
      {
        type: 'compareButton',
        dataElement: 'compareButton',
        element: 'compareButton',
        hidden: true,
      },
      {
        type: 'saveButton',
        dataElement: 'saveButton',
        element: 'saveButton',
        hidden: true,
      }
    ],
    rightHeaders: [
      {
        type: 'openFileButton',
        dataElement: 'openFileButton',
        element: 'openFileButton',
        title: 'Open File'
      },
      {
        type: 'searchButton',
        dataElement: 'searchButton',
        element: 'searchButton',
        title: 'Search'
      },
      {
        type: 'toggleRightPanelButton',
        img: 'icon-header-sidebar-line',
        element: 'rightPanel',
        dataElement: 'rightPanelButton',
        title: 'Right Panel',
        id: 'propertyPanelButton'
      },
      {
        type: 'downloadButton',
        dataElement: 'downloadButton',
        element: 'downloadButton',
        title: 'Download'
      },
      {
        type: 'flattenButton',
        dataElement: 'flattenButton',
        element: 'flattenButton',
        title: 'Save as Flattened PDF'
      },
      {
        type: 'printButton',
        dataElement: 'printButton',
        element: 'printButton',
        title: 'Print'
      }
    ],
    disabledElements: null,
    toolMode: 'view',
    tools: {
      annotation: [
        {
          type: 'markup',
          dataElement: 'markup',
          element: 'markup',
          hidden: false,
        }
      ],
      form: [
        {
          type: 'textFieldButton',
          dataElement: 'textFieldButton',
          element: 'textFieldButton',
          title: 'Form Field'
        },
        {
          type: 'checkBoxButton',
          dataElement: 'checkBoxButton',
          element: 'checkBoxButton',
          title: 'Check Box'
        },
        {
          type: 'radioButton',
          dataElement: 'radioButton',
          element: 'radioButton',
          title: 'Radio Button'
        },
        {
          type: 'listBox',
          dataElement: 'listBox',
          element: 'listBox',
          title: 'List Box'
        },
        {
          type: 'comboBox',
          dataElement: 'comboBox',
          element: 'comboBox',
          title: 'Combo Box'
        },
        {
          type: 'pushButton',
          dataElement: 'pushButton',
          element: 'pushButton',
          title: 'Push Button'
        }
      ],
      security: [
        {
          type: 'securitySelect',
          dataElement: 'securitySelect',
          element: 'securitySelect',
          title: 'Security'
        },
        {
          type: 'watermarkSelect',
          dataElement: 'watermarkSelect',
          element: 'watermarkSelect',
          title: 'Watermark'
        }
      ],
      compare: [
        {
          type: 'comparedToolbar'
        }
      ]
    },
    downloading: false,
    downloadError: '',
    activeSignCreatePanel: false,
    activeSignWay: 'trackpad',
    popoverChanged: false,
    dialogs: {
      compareSettingDialog: false,
      setPasswordModal: false
    },
    compareStatus: '',
    upload: true,
    uploadLoading: false
  }),
  getters: {
    getFullMode () {
      return this.fullMode
    },
    getCurrentPage () {
      return this.currentPage
    },
    getThemeMode () {
      return this.themeMode
    },
    getPageMode () {
      return this.pageMode
    },
    getScrollMode () {
      return this.scrollMode
    },
    getActiveHeaderItems () {
      return this.headers
    },
    getActiveRightHeaderItems () {
      return this.rightHeaders
    },
    isLeftPanelOpen: (state) => {
      return (dataElement) => state.openElements[dataElement] && !state.disabledElements?.[dataElement]?.disabled
    },
    isRightPanelOpen: (state) => {
      return (dataElement) => state.openElements[dataElement] && !state.disabledElements?.[dataElement]?.disabled
    },
    getActivePanel: (state) => {
      return (dataElement) => state.openElements[dataElement] && !state.disabledElements?.[dataElement]?.disabled
    },
    getActiveLeftPanel () {
      return this.activeLeftPanel
    },
    getActiveLeftPanelTab () {
      return this.activeLeftPanelTab
    },
    getActiveRightPanelTab () {
      return this.activeRightPanelTab
    },
    getActiveStampPanelTab () {
      return this.activeStampPanelTab
    },
    getScale () {
      return Math.round(this.scale * 100)
    },
    getActiveStickNote () {
      return this.activeStickNote
    },
    getActiveMeasure () {
      return this.activeActiveMeasure
    },
    getActiveHand () {
      return this.activeHand
    },
    getSearchStatus () {
      return this.searchStatus
    },
    getToolMode () {
      return this.toolMode
    },
    getToolItems () {
      return this.tools
    },
    getDownloading () {
      return this.downloading
    },
    getDownloadError () {
      return this.downloadError
    },
    getActiveSignCreatePanel () {
      return this.activeSignCreatePanel
    },
    getActiveSignWay () {
      return this.activeSignWay
    },
    getPopoverChanged () {
      return this.popoverChanged
    },
    getDialogState: (state) => {
      return (dialog) => state.dialogs[dialog]
    },
    getCompareStatus () {
      return this.compareStatus
    },
    getUpload () {
      return this.upload
    },
    getUploadLoading () {
      return this.uploadLoading
    }
  },
  actions: {
    resetSetting () {
      this.fullMode = false
      this.currentPage = 0
      this.scale = '',
      this.themeMode = 'Light'
      this.pageMode = 0
      this.scrollMode = 'Vertical'
      this.activeTab = 0
      this.activeLeftPanel = 'THUMBS',
      this.activeLeftPanelTab = 'THUMBS'
      this.activeRightPanelTab = 'GENERAL'
      this.activeStampPanelTab = 'STANDARD'
      this.searchStatus = false
      this.openElements = {
        header: true,
        leftPanel: false,
        rightPanel: false,
        stampPanel: false,
        linkPanel: false
      }
    },
    resetPanels () {
      this.searchStatus = false
      this.openElements = {
        header: true,
        leftPanel: false,
        rightPanel: false,
        stampPanel: false
      }
    },
    setFullMode (fullMode) {
      return this.fullMode = fullMode
    },
    setCurrentPage (currentPage) {
      this.currentPage = currentPage
    },
    setThemeMode (themeMode) {
      this.themeMode = themeMode
    },
    setPageMode (mode) {
      this.pageMode = mode
    },
    setScrollMode (mode) {
      this.scrollMode = mode
    },
    setCurrentScale (scale) {
      this.scale = scale
    },
    openElement (dataElement) {
      this.openElements[dataElement] = true
    },
    closeElement (dataElement) {
      this.openElements[dataElement] = false
    },
    toggleElement (dataElement) {
      if (this.disabledElements?.[dataElement]?.disabled) {
        return
      }
      if (this.openElements[dataElement]) {
        this.closeElement(dataElement)
      } else {
        this.openElement(dataElement)
      }
    },
    setActiveLeftPanel (dataElement) {
      this.activeLeftPanel = dataElement
    },
    setActiveLeftPanelTab (dataElement) {
      this.activeLeftPanelTab = dataElement
    },
    setActiveRightPanelTab (dataElement) {
      this.activeRightPanelTab = dataElement
    },
    setActiveStampPanelTab (dataElement) {
      this.activeStampPanelTab = dataElement
    },
    toggleActiveStickNote () {
      this.activeStickNote = !this.activeStickNote
    },
    closeActiveStickNote () {
      this.activeStickNote = false
    },
    toggleActiveMeasure () {
      this.activeActiveMeasure = !this.activeActiveMeasure
    },
    closeActiveMeasure () {
      this.activeActiveMeasure = false
    },
    toggleActiveHand (value) {
      this.activeHand = value
    },
    closeActiveHand () {
      this.activeHand = false
    },
    setSearchStatus (value) {
      this.searchStatus = value
    },
    setActiceToolMode (mode) {
      this.toolMode = mode
      this.openElements['stampPanel'] = false
      this.openElements['linkPanel'] = false
      core.setToolMode(mode)
    },
    setDownloading (boolean) {
      this.downloading = boolean
    },
    setDownloadError (text) {
      this.downloadError = text
    },
    toggleActiveSignCreatePanel () {
      this.activeSignCreatePanel = !this.activeSignCreatePanel
      if (this.activeSignCreatePanel) {
        this.setActiveSignWay('trackpad')
      }
    },
    setActiveSignWay (way) {
      this.activeSignWay = way
      if (this.activeSignWay === 'trackpad') {
        core.handleSign('create')
      }
    },
    setPopoverChanged (bool) {
      this.popoverChanged = bool
    },
    openDialog (dialog) {
      this.dialogs[dialog] = true
    },
    closeDialog (dialog) {
      this.dialogs[dialog] = false
    },
    setCompareStatus (string) {
      this.compareStatus = string
    },
    setUpload (bool) {
      this.upload = bool
    },
    setUploadLoading (bool) {
      this.uploadLoading = bool
    }
  }
})
